#define SIZEOF_PY_INTPTR_T 4
#define SIZEOF_PY_LONG_LONG 8
#define MATHLIB 
#define HAVE_SIN
#define HAVE_COS
#define HAVE_TAN
#define HAVE_SINH
#define HAVE_COSH
#define HAVE_TANH
#define HAVE_FABS
#define HAVE_FLOOR
#define HAVE_CEIL
#define HAVE_SQRT
#define HAVE_LOG10
#define HAVE_LOG
#define HAVE_EXP
#define HAVE_ASIN
#define HAVE_ACOS
#define HAVE_ATAN
#define HAVE_FMOD
#define HAVE_MODF
#define HAVE_FREXP
#define HAVE_LDEXP
#define HAVE_RINT
#define HAVE_TRUNC
#define HAVE_EXP2
#define HAVE_LOG2
#define HAVE_ATAN2
#define HAVE_POW
#define HAVE_NEXTAFTER
#define HAVE_SINF
#define HAVE_COSF
#define HAVE_TANF
#define HAVE_SINHF
#define HAVE_COSHF
#define HAVE_TANHF
#define HAVE_FABSF
#define HAVE_FLOORF
#define HAVE_CEILF
#define HAVE_RINTF
#define HAVE_TRUNCF
#define HAVE_SQRTF
#define HAVE_LOG10F
#define HAVE_LOGF
#define HAVE_LOG1PF
#define HAVE_EXPF
#define HAVE_EXPM1F
#define HAVE_ASINF
#define HAVE_ACOSF
#define HAVE_ATANF
#define HAVE_ASINHF
#define HAVE_ACOSHF
#define HAVE_ATANHF
#define HAVE_HYPOTF
#define HAVE_ATAN2F
#define HAVE_POWF
#define HAVE_FMODF
#define HAVE_MODFF
#define HAVE_FREXPF
#define HAVE_LDEXPF
#define HAVE_EXP2F
#define HAVE_LOG2F
#define HAVE_COPYSIGNF
#define HAVE_NEXTAFTERF
#define HAVE_SINL
#define HAVE_COSL
#define HAVE_TANL
#define HAVE_SINHL
#define HAVE_COSHL
#define HAVE_TANHL
#define HAVE_FABSL
#define HAVE_FLOORL
#define HAVE_CEILL
#define HAVE_RINTL
#define HAVE_TRUNCL
#define HAVE_SQRTL
#define HAVE_LOG10L
#define HAVE_LOGL
#define HAVE_LOG1PL
#define HAVE_EXPL
#define HAVE_EXPM1L
#define HAVE_ASINL
#define HAVE_ACOSL
#define HAVE_ATANL
#define HAVE_ASINHL
#define HAVE_ACOSHL
#define HAVE_ATANHL
#define HAVE_HYPOTL
#define HAVE_ATAN2L
#define HAVE_POWL
#define HAVE_FMODL
#define HAVE_MODFL
#define HAVE_FREXPL
#define HAVE_LDEXPL
#define HAVE_EXP2L
#define HAVE_LOG2L
#define HAVE_COPYSIGNL
#define HAVE_NEXTAFTERL
#define HAVE_DECL_SIGNBIT
#define HAVE_COMPLEX_H
#ifndef __cplusplus
/* #undef inline */
#endif

#ifndef _NPY_NPY_CONFIG_H_
#error config.h should never be included directly, include npy_config.h instead
#endif
